@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2008-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2008-2016 -- leonerd@leonerd.org.uk
 
 package IO::Termios;
 
@@ -11,7 +11,7 @@ use base qw( IO::Handle );
 
 use Carp;
 
-our $VERSION = '0.04';
+our $VERSION = '0.05';
 
 use Exporter 'import';
 
@@ -22,6 +22,9 @@ use IO::Tty::Constant qw(
    TIOCM_DTR TIOCM_DSR TIOCM_RTS TIOCM_CTS TIOCM_CD TIOCM_RI
 );
 
+# Linux can support finer-grained control of baud rates if we let it
+use constant HAVE_LINUX_TERMIOS2 => eval { require Linux::Termios2; };
+
 =head1 NAME
 
 C<IO::Termios> - supply F<termios(3)> methods to C<IO::Handle> objects
@@ -61,20 +64,33 @@ can be used to access the hardware handshaking lines.
  $ttyS0->set_mode( "19200,8,n,1" );
  $ttyS0->set_modem({ dsr => 1, cts => 1 });
 
+=head2 Arbitrary Baud Rates on Linux
+
+F<Linux> supports a non-POSIX extension to the usual C<termios> interface,
+which allows arbitrary baud rates to be set. C<IO::Termios> can automatically
+make use of this ability if the L<Linux::Termios2> module is installed. If so,
+this will be used automatically and transparently, to allow the C<set*baud>
+methods to set any rate allowed by the kernel/driver. If not, then only the
+POSIX-compatible rates may be used.
+
 =cut
 
 =head1 CONSTRUCTORS
 
 =cut
 
-=head2 $term = IO::Termios->new()
+=head2 new
+
+   $term = IO::Termios->new()
 
 Construct a new C<IO::Termios> object around the terminal for the program.
 This is found by checking if any of C<STDIN>, C<STDOUT> or C<STDERR> are a
 terminal. The first one that's found is used. An error occurs if no terminal
 can be found by this method.
 
-=head2 $term = IO::Termios->new( $handle )
+=head2 new (handle)
+
+   $term = IO::Termios->new( $handle )
 
 Construct a new C<IO::Termios> object around the given filehandle.
 
@@ -101,7 +117,9 @@ sub new
    return $self;
 }
 
-=head2 $term = IO::Termios->open( $path, $modestr )
+=head2 open
+
+   $term = IO::Termios->open( $path, $modestr )
 
 Open the given path, and return a new C<IO::Termios> object around the
 filehandle. If the C<open> call fails, C<undef> is returned.
@@ -128,7 +146,9 @@ sub open
 
 =cut
 
-=head2 $attrs = $term->getattr
+=head2 getattr
+
+   $attrs = $term->getattr
 
 Makes a C<tcgetattr()> call on the underlying filehandle, and returns a
 C<IO::Termios::Attrs> object.
@@ -147,7 +167,9 @@ sub getattr
    return $attrs;
 }
 
-=head2 $term->setattr( $attrs )
+=head2 setattr
+
+   $term->setattr( $attrs )
 
 Makes a C<tcsetattr()> call on the underlying file handle, setting attributes
 from the given C<IO::Termios::Attrs> object.
@@ -165,9 +187,13 @@ sub setattr
    return $attrs->setattr( $self->fileno, TCSANOW );
 }
 
-=head2 $term->set_mode( $modestr )
+=head2 set_mode
 
-=head2 $modestr = $term->get_mode
+=head2 get_mode
+
+   $term->set_mode( $modestr )
+
+   $modestr = $term->get_mode
 
 Accessor for the derived "mode string", which is a comma-joined concatenation
 of the baud rate, character size, parity mode, and stop size in a format such
@@ -209,9 +235,13 @@ sub get_mode
       $attrs->getstop;
 }
 
-=head2 $bits = $term->tiocmget
+=head2 tiocmget
 
-=head2 $term->tiocmset( $bits )
+=head2 tiocmset
+
+   $bits = $term->tiocmget
+
+   $term->tiocmset( $bits )
 
 Accessor for the modem line control bits. Takes or returns a bitmask of
 values.
@@ -239,13 +269,17 @@ sub tiocmset
       or croak "Cannot ioctl(TIOCMSET) - $!";
 }
 
-=head2 $term->tiombic( $bits )
+=head2 tiocmbic
 
-=head2 $term->tiombis( $bits )
+=head2 tiocmbis
 
-Bitwise mutator methods for the modem line control bits. C<tiombic> will clear
-just the bits provided and leave the others unchanged; C<tiombis> will set
-them.
+   $term->tiocmbic( $bits )
+
+   $term->tiocmbis( $bits )
+
+Bitwise mutator methods for the modem line control bits. C<tiocmbic> will
+clear just the bits provided and leave the others unchanged; C<tiocmbis> will
+set them.
 
 =cut
 
@@ -292,7 +326,9 @@ foreach (qw( dtr dsr rts cts cd ri )) {
    *{"setmodem_$_"} = $setmodem;
 }
 
-=head2 $flags = $term->get_modem
+=head2 get_modem
+
+   $flags = $term->get_modem
 
 Returns a hash reference containing named flags corresponding to the modem
 line control bits. Any bit that is set will yield a key in the returned hash
@@ -312,7 +348,9 @@ sub get_modem
    };
 }
 
-=head2 $term->set_modem( $flags )
+=head2 set_modem
+
+   $term->set_modem( $flags )
 
 Changes the modem line control bit flags as given by the hash reference. Each
 bit to be changed should be represented by a key in the C<$flags> hash of the
@@ -337,9 +375,13 @@ sub set_modem
    $self->tiocmset( $bits );
 }
 
-=head2 $set = $term->getmodem_BIT
+=head2 getmodem_BIT
 
-=head2 $term->setmodem_BIT( $set )
+=head2 setmodem_BIT
+
+   $set = $term->getmodem_BIT
+
+   $term->setmodem_BIT( $set )
 
 Accessor methods for each of the modem line control bits. A set of methods
 exists for each of the named modem control bits given above.
@@ -380,36 +422,58 @@ time. This is C<set_mode>:
 
 =cut
 
-=head2 $baud = $term->getibaud
+=head2 getibaud
 
-=head2 $baud = $term->getobaud
+=head2 getobaud
 
-=head2 $term->setibaud( $baud )
+=head2 setibaud
 
-=head2 $term->setobaud( $baud )
+=head2 setobaud
 
-=head2 $term->setbaud( $baud )
+=head2 setbaud
+
+   $baud = $term->getibaud
+
+   $baud = $term->getobaud
+
+   $term->setibaud( $baud )
+
+   $term->setobaud( $baud )
+
+   $term->setbaud( $baud )
 
 Convenience accessors for the C<ispeed> and C<ospeed>. C<$baud> is an integer
 directly giving the line rate, instead of one of the C<BI<nnn>> constants.
 
-=head2 $bits = $term->getcsize
+=head2 getcsize
 
-=head2 $term->setcsize( $bits )
+=head2 setcsize
+
+   $bits = $term->getcsize
+
+   $term->setcsize( $bits )
 
 Convenience accessor for the C<CSIZE> bits of C<c_cflag>. C<$bits> is an
 integer 5 to 8.
 
-=head2 $parity = $term->getparity
+=head2 getparity
 
-=head2 $term->setparity( $parity )
+=head2 setparity
+
+   $parity = $term->getparity
+
+   $term->setparity( $parity )
 
 Convenience accessor for the C<PARENB> and C<PARODD> bits of C<c_cflag>.
 C<$parity> is C<n>, C<o> or C<e>.
 
-=head2 $stop = $term->getstop
+=head2 getstop
 
-=head2 $term->setstop( $stop )
+=head2 setstop
+
+   $stop = $term->getstop
+
+   $term->setstop( $stop )
 
 Convenience accessor for the C<CSTOPB> bit of C<c_cflag>. C<$stop> is 1 or 2.
 
@@ -440,42 +504,62 @@ foreach my $name (qw( ibaud obaud csize parity stop )) {
    $self->setattr( $attrs ) or croak "Cannot setattr - $!";
 };
 
-=head2 $mode = $term->getflag_cread
+=head2 getflag_cread
 
-=head2 $term->setflag_cread( $mode )
+=head2 setflag_cread
+
+   $mode = $term->getflag_cread
+
+   $term->setflag_cread( $mode )
 
 Accessor for the C<CREAD> bit of the C<c_cflag>. This enables the receiver.
 
-=head2 $mode = $term->getflag_hupcl
+=head2 getflag_hupcl
 
-=head2 $term->setflag_hupcl( $mode )
+=head2 setflag_hupcl
+
+   $mode = $term->getflag_hupcl
+
+   $term->setflag_hupcl( $mode )
 
 Accessor for the C<HUPCL> bit of the C<c_cflag>. This lowers the modem control
 lines after the last process closes the device.
 
-=head2 $mode = $term->getflag_clocal
+=head2 getflag_clocal
 
-=head2 $term->setflag_clocal( $mode )
+=head2 setflag_clocal
+
+   $mode = $term->getflag_clocal
+
+   $term->setflag_clocal( $mode )
 
 Accessor for the C<CLOCAL> bit of the C<c_cflag>. This controls whether local
 mode is enabled; which if set, ignores modem control lines.
 
 =cut
 
-=head2 $mode = $term->getflag_icanon
+=head2 getflag_icanon
 
-=head2 $term->setflag_icanon( $mode )
+=head2 setflag_icanon
+
+   $mode = $term->getflag_icanon
+
+   $term->setflag_icanon( $mode )
 
 Accessor for the C<ICANON> bit of C<c_lflag>. This is called "canonical" mode
 and controls whether the terminal's line-editing feature will be used to
-return a whole line (if false), or if individual bytes from keystrokes will be
-returned as they are available (if true).
+return a whole line (if true), or if individual bytes from keystrokes will be
+returned as they are available (if false).
 
 =cut
 
-=head2 $mode = $term->getflag_echo
+=head2 getflag_echo
 
-=head2 $term->setflag_echo( $mode )
+=head2 setflag_echo
+
+   $mode = $term->getflag_echo
+
+   $term->setflag_echo( $mode )
 
 Accessor for the C<ECHO> bit of C<c_lflag>. This controls whether input
 characters are echoed back to the terminal.
@@ -515,14 +599,12 @@ foreach ( @flags ) {
 package # hide from CPAN
    IO::Termios::Attrs;
 
-use base qw( POSIX::Termios );
-
 use Carp;
 use POSIX qw( CSIZE CS5 CS6 CS7 CS8 PARENB PARODD CSTOPB );
 # IO::Tty has more B<\d> constants than POSIX has
 use IO::Tty;
 
-# POSIX::Termios does not respect subclassing
+# Simple XS-implemented classes tend not to respect subclassing
 sub new
 {
    my $class = shift;
@@ -531,6 +613,42 @@ sub new
    return $self;
 }
 
+if( IO::Termios::HAVE_LINUX_TERMIOS2 ) {
+   our @ISA = qw( Linux::Termios2 );
+
+   # baud is directly applicable
+   *getibaud = __PACKAGE__->can( 'getispeed' );
+   *getobaud = __PACKAGE__->can( 'getospeed' );
+
+   *setibaud = __PACKAGE__->can( 'setispeed' );
+   *setobaud = __PACKAGE__->can( 'setospeed' );
+}
+else {
+   our @ISA = qw( POSIX::Termios );
+
+   # baud needs converting to/from the speed_t constants
+
+   my %_speed2baud = map { IO::Tty::Constant->${\"B$_"} => $_ } 
+      qw( 0 50 75 110 134 150 200 300 600 1200 2400 4800 9600 19200 38400 57600 115200 230400 );
+   my %_baud2speed = reverse %_speed2baud;
+
+   *getibaud = sub { $_speed2baud{ $_[0]->getispeed } };
+   *getobaud = sub { $_speed2baud{ $_[0]->getospeed } };
+
+   *setibaud = sub {
+      $_[0]->setispeed( $_baud2speed{$_[1]} // die "Unrecognised baud rate" );
+   };
+   *setobaud = sub {
+      $_[0]->setospeed( $_baud2speed{$_[1]} // die "Unrecognised baud rate" );
+   };
+
+}
+
+sub setbaud
+{
+   $_[0]->setibaud( $_[1] ); $_[0]->setobaud( $_[1] );
+}
+
 foreach ( @flags ) {
    my ( $name, $const, $member ) = @$_;
 
@@ -554,22 +672,6 @@ foreach ( @flags ) {
    };
 }
 
-my %_speed2baud = map { IO::Tty::Constant->${\"B$_"} => $_ } 
-   qw( 0 50 75 110 134 150 200 300 600 1200 2400 4800 9600 19200 38400 57600 115200 230400 );
-my %_baud2speed = reverse %_speed2baud;
-
-sub getibaud { $_speed2baud{ $_[0]->getispeed } }
-sub getobaud { $_speed2baud{ $_[0]->getospeed } }
-
-sub setibaud { $_[0]->setispeed( $_baud2speed{$_[1]} ) }
-sub setobaud { $_[0]->setospeed( $_baud2speed{$_[1]} ) }
-
-sub setbaud
-{
-   my $speed = $_baud2speed{$_[1]};
-   $_[0]->setispeed( $speed ) and $_[0]->setospeed( $speed );
-}
-
 sub getcsize
 {
    my $self = shift;
